#include <stdio.h>

#include <avr/io.h>
#include <avr/interrupt.h>

#include <motelib/timer.h>
#include <motelib/system.h>
#include <motelib/led.h>
#include <motelib/uart.h>
#include <motelib/radio.h>
#include <motelib/sensor.h>
#include <motelib/button.h>
#include <motelib/actor.h>

#include <util/delay.h>
#define SPINSTATION_ID  0
#define RAIN_IS_COMING  1
#define RAIN_IS_LEAVING 2

#define ntohs(x) (((x&0xff00)>>8)|((x&0x00ff)<<8))
#define htons(x) (((x&0xff00)>>8)|((x&0x00ff)<<8))

Timer t;
Timer t2;
int roundSpin;
struct{
    uint8_t type;
    uint8_t seq;
}message;

uint16_t value;
void readRain(Timer *t);
uint8_t rain_stats = RAIN_IS_LEAVING;


uint16_t read_adc_2(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

void dataReady(uint16_t rain_v) // rain_v = read value
{
/*
    message.value = rain_v;
    message.address = getAddress();
    message.value = htons(message.value);
    message.address = htons(message.address);
    radioRequestTx(BROADCAST_ADDR,0 , &message, sizeof(message), NULL);
    message.value = ntohs(message.value);
    message.address = ntohs(message.address);
*/
    printf("my name is rain: %d\n",rain_v);
}

void button(ButtonStatus s)
{
    //uint8_t x = 0;
    //uint16_t x = 0;
    if (s == BUTTON_PRESSED)
    {
        //actorSetState(ACTOR_0,1); //send high --> PC0
        //sensorRequestAnalog(SENSOR_0, dataReady);
        //sensorRequestAnalog(SENSOR_1, dataReady);
        //printf("%d\n",sensorAnalogResult(SENSOR_2));
        //sensorRequestAnalog(SENSOR_3, dataReady);
        //sensorRequestAnalog(SENSOR_4, dataReady);
        //sensorRequestAnalog(SENSOR_5, dataReady);


        ledSetValue(4);
    }
    if (s == BUTTON_RELEASED)
    {
        //x = sensorReadDigital(SENSOR_2);
	//printf("x = %d\n",x);
        //actorSetState(ACTOR_0,0); //stop send signal to PC0
        ledSetValue(0);
    }
}
void readRain(Timer *t)
{
    //sensorRequestAnalog(SENSOR_2, dataReady);
    //printf("my name is RAIN: %d\n",read_adc_2(2));
    if(read_adc_2(SENSOR_2) <= 800 && rain_stats == RAIN_IS_LEAVING)
    {
		rain_stats = RAIN_IS_COMING;
        message.seq += 1;
		message.type = RAIN_IS_COMING;
        //message.type = htons(RAIN_IS_COMING);
        //message.seq = htons(message.seq);
        radioRequestTx(BROADCAST_ADDR ,0 ,&message, sizeof(message), NULL);
		printf("SEND !\n");
        //message.seq = ntohs(message.seq);
        //message.type = ntohs(message.type);
    }
	if(read_adc_2(SENSOR_2) >= 900 && rain_stats == RAIN_IS_COMING)
	{
		rain_stats = RAIN_IS_LEAVING;
        message.seq += 1;
		message.type = RAIN_IS_LEAVING;
        //message.type = htons(RAIN_IS_LEAVING);
        //message.seq = htons(message.seq);
        radioRequestTx(BROADCAST_ADDR ,0 ,&message, sizeof(message), NULL);
        //message.seq = ntohs(message.seq);
        //message.type = ntohs(message.type);
	}
}

void receive(Address source,MessageType m_type,void *msg,uint8_t len)
{
    uint8_t type = ((uint8_t*)msg)[0];//ntohs(((uint8_t*)msg)[0]);
    uint8_t seq = ((uint8_t*)msg)[1];//ntohs((uint8_t*)msg)[1];
    printf("%d %d\n",type,seq);
	if(seq > message.seq)
	{
    	    //radioRequestTx(BROADCAST_ADDR ,0 ,&message, sizeof(message), NULL);
	    if(type == RAIN_IS_COMING)
	    {
		    roundSpin = 6;
	    }
	}
}

void spin(Timer *t2)
{
    if(roundSpin !=0 )
    {
        DDRC = 0x01;
        PORTC = 0x01;
        roundSpin--;
    }
    else if(roundSpin == 0)
    {
        PORTC = 0x00;
    }

}

void boot()
{
    buttonSetHandler(button);
    radioSetRxHandler(receive);
    uartEnable(true,true);
    if(getAddress() == SPINSTATION_ID)
    {
        timerCreate(&t2);
        timerStart(&t2, TIMER_PERIODIC, 500, spin);
    }
    else
    {
        timerCreate(&t);
        timerStart(&t, TIMER_PERIODIC, 500, readRain);
    }
	message.seq = 0;
}
